//=================================
// include guard
#pragma once

//=================================
// forward declared dependencies
class particle;

//=================================
// included dependencies
#include "particle.h"
#include <armadillo>
#include "..\enumerations.h"
//#include "..\ITL_networks\General_Functions.h"

//=================================
// namespaces
using namespace std;
using namespace arma;

//=================================
// type that defines pointer to comparison funciton depending whether the problem
// is minimization or maximization
typedef bool (* CompareFunction)(particle&, particle&); 


//=================================
// the fitnessfunction class
class fitnessfunction
{
private:
	bool Minimize; // sense of optimization - maximize or minimize

public:

	fitnessfunction(void);
	//fitnessfunction(unsigned int newDim);
	// ~fitnessfunction(void);

	CompareFunction particleCompare; // pointer to predicate function for comparison of two particles, used for sort
	
	//the most important member function is the following 
	// takes the vector and returns the value of fitness function double 
	// needs to be redefined in inherited classes

	virtual double calcFitness(   
		const fvec &variaveisProblema,  
		vector <double> X,
		const double &minPosMath,
		const double &maxPosMath,
		const mat &W1, const mat &W2,
		const rowvec &B1, const rowvec &B2, 
		const rowvec &Parameter1, const rowvec &Parameter2,
		const float &actFcnFlatness, const ActivationFunctions &NeuronActFunct2, const NormalizationMethod &NormMethod) = 0;

	void SetMinimize(bool MinimizeFlag); // sets the flag regarding the optimization sense- minimize or maximize
	bool fitnessCompare(double fit1, double fit2); // comparison predicate function for doubles
};


bool particle_compare_minimize(particle &pP1, particle &pP2); // comparison predicate function - minimization
bool particle_compare_maximize(particle &pP1, particle &pP2); // comparison predicate function - maximization
