//=================================
// include guard
#pragma once

//=================================
// included dependencies from EPSO small
#include "fitnessfunction.h"

//=================================
// included dependencies from ITL networks
#include "..\ITL_networks\General_Functions.h"
#include "..\ITL_networks\Normalization_functions.h"
#include "..\enumerations.h"

//=================================
// included dependencies
#include <armadillo>
#include <stdio.h>
#include <math.h>

#ifndef PI
#define PI 3.14159265
#endif
//=================================
// included namespaces
using namespace std;
using namespace arma;

//=================================

//######################################################################
// MATH FUNCTIONS
//######################################################################


// Rosenbrock
class RosenbrockS : public fitnessfunction
{
public:
	RosenbrockS():fitnessfunction(){ }; // constructor

	double calcFitness(
		const fvec &variaveisProblema,   
		vector<double> X,
		const double &minPosMath,
		const double &maxPosMath,
		const mat &W1, const mat &W2,
		const rowvec &B1, const rowvec &B2, 
		const rowvec &Parameter1, const rowvec &Parameter2,
		const float &actFcnFlatness, const ActivationFunctions &NeuronActFunct2, const NormalizationMethod &NormMethod)
	{
		
		unsigned int DimHidden = variaveisProblema (1);
		unsigned int DimIn = variaveisProblema(0);

		//=================================
		//expansion and desnormalization
		mat PatternOriginal (1,DimHidden);
		for (unsigned int k=0; k<DimHidden; k++)
		{
			PatternOriginal (0,k) = X[k];
		}
		unsigned short numPatternsDesteCaso=1;

		mat PatternExpandido (1,DimIn);
		General_functions::transitionLayer (PatternExpandido, numPatternsDesteCaso, DimHidden, DimIn, PatternOriginal, W2, B2, actFcnFlatness, NeuronActFunct2);

		mat PatternDESnormalized (1,DimIn);

		PatternDESnormalized = Normalization::DES_Normalization_function (Parameter1, Parameter2, PatternExpandido, numPatternsDesteCaso, DimIn, NormMethod);

		vector <double> Pos (DimIn); 
		for (unsigned int k=0; k<DimIn; k++)
		{
			Pos[k]= PatternDESnormalized(0,k);

			if (Pos[k]<minPosMath){Pos[k]=minPosMath;}
			if (Pos[k]>maxPosMath){Pos[k]=maxPosMath;}
		}

		double fit = 0.0;
						
		for (unsigned int i=0; i<(DimIn-1); i++) 
		{
			fit=fit + (100*pow((Pos[i+1]-Pos[i]*Pos[i]),2)+pow((Pos[i]-1),2));	
		}
		return fit;
	}
};

//-----------------------------------------------------------------------------------------------------------------------------------------
// Sphere
class SphereS : public fitnessfunction
{
public:
	SphereS():fitnessfunction(){ }; // constructor

	double calcFitness(
		const fvec &variaveisProblema,   
		vector<double> X,
		const double &minPosMath,
		const double &maxPosMath,
		const mat &W1, const mat &W2,
		const rowvec &B1, const rowvec &B2, 
		const rowvec &Parameter1, const rowvec &Parameter2,
		const float &actFcnFlatness, const ActivationFunctions &NeuronActFunct2, const NormalizationMethod &NormMethod) 
	{
		unsigned int DimHidden = variaveisProblema (1);
		unsigned int DimIn = variaveisProblema(0);

		//=================================
		//expansion and desnormalization
		mat PatternOriginal (1,DimHidden);
		for (unsigned int k=0; k<DimHidden; k++)
		{
			PatternOriginal (0,k) = X[k];
		}
		unsigned short numPatternsDesteCaso=1;

		mat PatternExpandido (1,DimIn);
		General_functions::transitionLayer (PatternExpandido, numPatternsDesteCaso, DimHidden, DimIn, PatternOriginal, W2, B2, actFcnFlatness, NeuronActFunct2);

		mat PatternDESnormalized (1,DimIn);

		PatternDESnormalized = Normalization::DES_Normalization_function (Parameter1, Parameter2, PatternExpandido, numPatternsDesteCaso, DimIn, NormMethod);

		vector <double> Pos (DimIn); 
		for (unsigned int k=0; k<DimIn; k++)
		{
			Pos[k]= PatternDESnormalized(0,k);

			if (Pos[k]<minPosMath){Pos[k]=minPosMath;}
			if (Pos[k]>maxPosMath){Pos[k]=maxPosMath;}
		}

		double fit = 0.0;


		for (unsigned int i=0; i<(DimIn); i++)
		{
			fit=fit+ (Pos[i] - 1.0)*(Pos[i] - 1.0); // sphere, sum (xi - 1)^2
		}
		return fit;
	}
};

//-----------------------------------------------------------------------------------------------------------------------------------------
// Shifted Sphere
class ShifSphereS : public fitnessfunction
{
public:
	ShifSphereS():fitnessfunction(){ }; // constructor

	double calcFitness(
		const fvec &variaveisProblema,   
		vector<double> X,
		const double &minPosMath,
		const double &maxPosMath,
		const mat &W1, const mat &W2,
		const rowvec &B1, const rowvec &B2, 
		const rowvec &Parameter1, const rowvec &Parameter2,
		const float &actFcnFlatness, const ActivationFunctions &NeuronActFunct2, const NormalizationMethod &NormMethod) 
	{
		unsigned int DimHidden = variaveisProblema (1);
		unsigned int DimIn = variaveisProblema(0);

		//=================================
		//expansion and desnormalization
		mat PatternOriginal (1,DimHidden);
		for (unsigned int k=0; k<DimHidden; k++)
		{
			PatternOriginal (0,k) = X[k];
		}
		unsigned short numPatternsDesteCaso=1;

		mat PatternExpandido (1,DimIn);
		General_functions::transitionLayer (PatternExpandido, numPatternsDesteCaso, DimHidden, DimIn, PatternOriginal, W2, B2, actFcnFlatness, NeuronActFunct2);

		mat PatternDESnormalized (1,DimIn);

		PatternDESnormalized = Normalization::DES_Normalization_function (Parameter1, Parameter2, PatternExpandido, numPatternsDesteCaso, DimIn, NormMethod);

		vector <double> Pos (DimIn); 
		for (unsigned int k=0; k<DimIn; k++)
		{
			Pos[k]= PatternDESnormalized(0,k);

			if (Pos[k]<minPosMath){Pos[k]=minPosMath;}
			if (Pos[k]>maxPosMath){Pos[k]=maxPosMath;}
		}

		double fit = 0.0;
		
		for (unsigned int i=0; i<(DimIn); i++)
		{
			fit=fit+ (Pos[i] - 1.0)*(Pos[i] - 1.0); // sphere, sum (xi - 1)^2
		}
		fit =fit - 450.0;

		return fit;
	}
};


//-----------------------------------------------------------------------------------------------------------------------------------------
// Shifted Rastrigin
class ShifRastriginS : public fitnessfunction
{
public:
	ShifRastriginS():fitnessfunction(){ }; // constructor

	double calcFitness(
		const fvec &variaveisProblema,   
		vector<double> X,
		const double &minPosMath,
		const double &maxPosMath,
		const mat &W1, const mat &W2,
		const rowvec &B1, const rowvec &B2, 
		const rowvec &Parameter1, const rowvec &Parameter2,
		const float &actFcnFlatness, const ActivationFunctions &NeuronActFunct2, const NormalizationMethod &NormMethod) 
	{
		unsigned int DimHidden = variaveisProblema (1);
		unsigned int DimIn = variaveisProblema(0);

		//=================================
		//expansion and desnormalization
		mat PatternOriginal (1,DimHidden);
		for (unsigned int k=0; k<DimHidden; k++)
		{
			PatternOriginal (0,k) = X[k];
		}
		unsigned short numPatternsDesteCaso=1;

		mat PatternExpandido (1,DimIn);
		General_functions::transitionLayer (PatternExpandido, numPatternsDesteCaso, DimHidden, DimIn, PatternOriginal, W2, B2, actFcnFlatness, NeuronActFunct2);

		mat PatternDESnormalized (1,DimIn);

		PatternDESnormalized = Normalization::DES_Normalization_function (Parameter1, Parameter2, PatternExpandido, numPatternsDesteCaso, DimIn, NormMethod);

		vector <double> Pos (DimIn); 
		for (unsigned int k=0; k<DimIn; k++)
		{
			Pos[k]= PatternDESnormalized(0,k);

			if (Pos[k]<minPosMath){Pos[k]=minPosMath;}
			if (Pos[k]>maxPosMath){Pos[k]=maxPosMath;}
		}

		double fit = 0.0;
		
		for (unsigned int i=0; i<(DimIn); i++)
		{
			fit=fit + (Pos[i]*Pos[i] - 10.0*cos(2.0*PI*Pos[i])+10.0);
		}
		fit =fit - 330.0;

		return fit;
	}
};



//-----------------------------------------------------------------------------------------------------------------------------------------
// Shifted Schwefel
class ShifSchwefelS : public fitnessfunction
{
public:
	ShifSchwefelS():fitnessfunction(){ }; // constructor

	double calcFitness(
		const fvec &variaveisProblema,   
		vector<double> X,
		const double &minPosMath,
		const double &maxPosMath,
		const mat &W1, const mat &W2,
		const rowvec &B1, const rowvec &B2, 
		const rowvec &Parameter1, const rowvec &Parameter2,
		const float &actFcnFlatness, const ActivationFunctions &NeuronActFunct2, const NormalizationMethod &NormMethod) 
	{
		unsigned int DimHidden = variaveisProblema (1);
		unsigned int DimIn = variaveisProblema(0);

		//=================================
		//expansion and desnormalization
		mat PatternOriginal (1,DimHidden);
		for (unsigned int k=0; k<DimHidden; k++)
		{
			PatternOriginal (0,k) = X[k];
		}
		unsigned short numPatternsDesteCaso=1;

		mat PatternExpandido (1,DimIn);
		General_functions::transitionLayer (PatternExpandido, numPatternsDesteCaso, DimHidden, DimIn, PatternOriginal, W2, B2, actFcnFlatness, NeuronActFunct2);

		mat PatternDESnormalized (1,DimIn);

		PatternDESnormalized = Normalization::DES_Normalization_function (Parameter1, Parameter2, PatternExpandido, numPatternsDesteCaso, DimIn, NormMethod);

		vector <double> Pos (DimIn); 
		for (unsigned int k=0; k<DimIn; k++)
		{
			Pos[k]= PatternDESnormalized(0,k);

			if (Pos[k]<minPosMath){Pos[k]=minPosMath;}
			if (Pos[k]>maxPosMath){Pos[k]=maxPosMath;}
		}

		double fit = 0.0;
		double sumIn = 0.0;

		for (unsigned int i=0; i<(DimIn); i++)
		{
			for (unsigned int j=0; j<i; j++)
			{
				sumIn=sumIn+Pos[j];  //!!!!!!!!!
			}
			fit = fit+sumIn*sumIn;

			sumIn=0.0;
		}

		fit = fit - 450.0; //fbias=-450, the optimum fitness value is -450
		return fit;
	}
};




//--------------------------------------------------------------------------------------
class AlpineS : public fitnessfunction
{
public:
	AlpineS():fitnessfunction(){ }; // constructor

	double calcFitness(
		const fvec &variaveisProblema,   
		vector<double> X,
		const double &minPosMath,
		const double &maxPosMath,
		const mat &W1, const mat &W2,
		const rowvec &B1, const rowvec &B2, 
		const rowvec &Parameter1, const rowvec &Parameter2,
		const float &actFcnFlatness, const ActivationFunctions &NeuronActFunct2, const NormalizationMethod &NormMethod)
	{
		unsigned int DimHidden = variaveisProblema (1);
		unsigned int DimIn = variaveisProblema(0);

		//=================================
		//expansion and desnormalization
		mat PatternOriginal (1,DimHidden);
		for (unsigned int k=0; k<DimHidden; k++)
		{
			PatternOriginal (0,k) = X[k];
		}
		unsigned short numPatternsDesteCaso=1;

		mat PatternExpandido (1,DimIn);
		General_functions::transitionLayer (PatternExpandido, numPatternsDesteCaso, DimHidden, DimIn, PatternOriginal, W2, B2, actFcnFlatness, NeuronActFunct2);

		mat PatternDESnormalized (1,DimIn);

		PatternDESnormalized = Normalization::DES_Normalization_function (Parameter1, Parameter2, PatternExpandido, numPatternsDesteCaso, DimIn, NormMethod);

		vector <double> Pos (DimIn); 
		for (unsigned int k=0; k<DimIn; k++)
		{
			Pos[k]= PatternDESnormalized(0,k);

			if (Pos[k]<minPosMath){Pos[k]=minPosMath;}
			if (Pos[k]>maxPosMath){Pos[k]=maxPosMath;}
		}

		double fit = 1.0;
		double mult=1.0;
		
		for (unsigned int i=0; i<(DimIn); i++) 
		{
			fit=fit * sin(Pos[i]);
			mult = mult*Pos[i];
		}
		fit=fit*sqrt(mult);

		return fit;
	}
};


//-------------------------------------------------------------------------
class SchafferS : public fitnessfunction
{
public:
	SchafferS():fitnessfunction(){ }; // constructor

	double calcFitness(
		const fvec &variaveisProblema,   
		vector<double> X,
		const double &minPosMath,
		const double &maxPosMath,
		const mat &W1, const mat &W2,
		const rowvec &B1, const rowvec &B2, 
		const rowvec &Parameter1, const rowvec &Parameter2,
		const float &actFcnFlatness, const ActivationFunctions &NeuronActFunct2, const NormalizationMethod &NormMethod)
	{
		unsigned int DimHidden = variaveisProblema (1);
		unsigned int DimIn = variaveisProblema(0);

		//=================================
		//expansion and desnormalization
		mat PatternOriginal (1,DimHidden);
		for (unsigned int k=0; k<DimHidden; k++)
		{
			PatternOriginal (0,k) = X[k];
		}
		unsigned short numPatternsDesteCaso=1;

		mat PatternExpandido (1,DimIn);
		General_functions::transitionLayer (PatternExpandido, numPatternsDesteCaso, DimHidden, DimIn, PatternOriginal, W2, B2, actFcnFlatness, NeuronActFunct2);

		mat PatternDESnormalized (1,DimIn);

		PatternDESnormalized = Normalization::DES_Normalization_function (Parameter1, Parameter2, PatternExpandido, numPatternsDesteCaso, DimIn, NormMethod);

		vector <double> Pos (DimIn); 
		for (unsigned int k=0; k<DimIn; k++)
		{
			Pos[k]= PatternDESnormalized(0,k);

			if (Pos[k]<minPosMath){Pos[k]=minPosMath;}
			if (Pos[k]>maxPosMath){Pos[k]=maxPosMath;}
		}

		double fit = 0.0;

		// s usa a posio x e y, as outras no interessam... 
		double x1 = Pos[0];
		double x2 = Pos[1];

		fit = 0.5 + ((std::sin(std::sqrt(x1*x1+x2*x2)))*(std::sin(std::sqrt(x1*x1+x2*x2))) - 0.5) / ((1.0 + 0.001*(x1*x1+x2*x2))*(1.0 + 0.001*(x1*x1+x2*x2)));

		return fit;
	}
};


//---------------------------------------------------------------------------------
class GriewankS : public fitnessfunction
{
public:
	GriewankS():fitnessfunction(){ }; // constructor

	double calcFitness(
		const fvec &variaveisProblema,   
		vector<double> X,
		const double &minPosMath,
		const double &maxPosMath,
		const mat &W1, const mat &W2,
		const rowvec &B1, const rowvec &B2, 
		const rowvec &Parameter1, const rowvec &Parameter2,
		const float &actFcnFlatness, const ActivationFunctions &NeuronActFunct2, const NormalizationMethod &NormMethod)

	{
		unsigned int DimHidden = variaveisProblema (1);
		unsigned int DimIn = variaveisProblema(0);

		//=================================
		//expansion and desnormalization
		mat PatternOriginal (1,DimHidden);
		for (unsigned int k=0; k<DimHidden; k++)
		{
			PatternOriginal (0,k) = X[k];
		}
		unsigned short numPatternsDesteCaso=1;

		mat PatternExpandido (1,DimIn);
		General_functions::transitionLayer (PatternExpandido, numPatternsDesteCaso, DimHidden, DimIn, PatternOriginal, W2, B2, actFcnFlatness, NeuronActFunct2);

		mat PatternDESnormalized (1,DimIn);

		PatternDESnormalized = Normalization::DES_Normalization_function (Parameter1, Parameter2, PatternExpandido, numPatternsDesteCaso, DimIn, NormMethod);

		vector <double> Pos (DimIn); 
		for (unsigned int k=0; k<DimIn; k++)
		{
			Pos[k]= PatternDESnormalized(0,k);

			if (Pos[k]<minPosMath){Pos[k]=minPosMath;}
			if (Pos[k]>maxPosMath){Pos[k]=maxPosMath;}
		}

		double fit = 0.0;
		double p = 1.0;
		for (unsigned int i=0; i<(DimIn); i++)
		{  
			fit = fit + Pos[i]*Pos[i];
			p = p * (cos(Pos[i]/sqrt((double) i+1)));
		}
		fit = fit / 4000 - p + 1;
		return fit;
	}
};
