#include "particle.h"

//=================================
// destructor
particle::~particle(void){}

//=================================
// constructor, default
particle::particle(void){}

//=================================
// Constructor for particles with position and velocity known in advance
particle::particle(
	const fvec &variaveisProblema,   
	const double &minPosMath, const double &maxPosMath,
	vector<double> tempPos,
	vector<double> tempBeforePos,
	vector<double> tempBestPos,
	const mat &W1, const mat &W2, const rowvec &B1, const rowvec &B2, 
	const rowvec &Parameter1, const rowvec &Parameter2,
	const float &actFcnFlatness,
	const ActivationFunctions &NeuronActFunct1, const ActivationFunctions &NeuronActFunct2,
	const NormalizationMethod &NormMethod,
	double inertiaParteB, double memoryParteB, double cooperationParteB, double perturbationParteB,
	boost::mt19937& rng,
	fitnessfunction *pFf, 
	const double &CpP,
	const Initi_Vel &Vel_init)
{
	this->inertiaParteB=inertiaParteB;
	this->memoryParteB=memoryParteB;
	this->cooperationParteB=cooperationParteB;
	this->perturbationParteB=perturbationParteB;
	
	unsigned int DimIn=variaveisProblema(0);
	unsigned int DimHidden = variaveisProblema(1);

	unsigned short numPatterns = 3;//tempPos + tempBestPos + tempBeforePos

	mat notNormalizedPos (numPatterns,DimIn);
	for (unsigned int k=0; k<DimIn; k++)
	{
		notNormalizedPos(0,k)  = tempPos[k];//position from moment t
		notNormalizedPos(1,k)  = tempBestPos[k]; //my Best position from moment t
		notNormalizedPos(2,k)  = tempBeforePos[k]; //position from the moment t-1
	}

	
	mat  NormalizedPos =  Normalization::Normalization_function (
		Parameter1, Parameter2,  
		notNormalizedPos, 
		numPatterns, 
		DimIn, 
		NormMethod);

	mat CompressedPos(numPatterns,DimIn);

	General_functions::transitionLayer (
		CompressedPos,
		numPatterns, 
		DimIn, 
		DimHidden, 
		NormalizedPos, 
		W1, 
		B1, 
		actFcnFlatness, 
		NeuronActFunct1);

	this->ff = pFf;

	//assim o domnio so hipercubos, onde em cada dimenso os limites so -1 e 1
	double delta=0.0;
	double minpos=-1+delta;
	double maxpos=1-delta;
	for (unsigned int i=0; i<DimHidden; i++)
	{
		minPos.push_back(minpos);
		maxPos.push_back(maxpos);

		minVel.push_back(-fabs(maxpos - minpos));//-2
		maxVel.push_back(fabs(maxpos - minpos));//2

		//minVel.push_back(-1);
		//maxVel.push_back(1);
	}

	this->maxPos=maxPos;
	this->minPos=minPos;
	this->minVel=minVel;
	this->maxVel=maxVel;

	boost::uniform_real<double> uniform_weight(-1,1);//NOTE:: before vel was with (-2,2)
	boost::variate_generator<boost::mt19937&, boost::uniform_real<double> > uniform_sampler_weights(rng, uniform_weight);

	for (unsigned int i=0; i<DimHidden; i++) //aqui o dim vai ser numS
    {
        this->Pos.push_back(CompressedPos(0,i));
       
		
		this->myBestPos.push_back(CompressedPos(1,i));//mybest pos

		switch (Vel_init)
		{ 
			case RANDOM_VEL: 
				{
					//velocities are initialized as random
					this->Vel.push_back(uniform_sampler_weights());
					break;
				}
			case TRANSFER_VEL: 
				{
					//velocities are initialize as the diference V(k) = X(k)-X(k-1) 
					this->Vel.push_back(CompressedPos(0,i)-CompressedPos(2,i));
					break;
				}
			default: cout << "ERROR!!! ";
		}


    }
	
	
	this->UpdateFitness(
		variaveisProblema,  
		minPosMath,
		maxPosMath, 
		W1, W2, B1, B2, Parameter1, Parameter2, 
		actFcnFlatness, NeuronActFunct2, NormMethod);
	myBestFitness=Fitness;
}

// ----------------------------------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------------------------------


double particle::GetFitness()
{
	return Fitness;
}

void particle::UpdateFitness(
	const fvec &variaveisProblema,   
	const double &minPosMath,
	const double &maxPosMath,
	const mat &W1, const mat &W2,
	const rowvec &B1, const rowvec &B2,  const rowvec &Parameter1, const rowvec &Parameter2,
	const float &actFcnFlatness, 
	const ActivationFunctions &NeuronActFunct2,
	const NormalizationMethod &NormMethod)
{
	this->Fitness=this->ff->calcFitness(
		variaveisProblema, 
		Pos,
		minPosMath,
		maxPosMath, 
		 W1, W2, B1, B2, Parameter1, Parameter2, actFcnFlatness,
		NeuronActFunct2, NormMethod);
}


void particle::Print()
{
	//unsigned int i;
	//for (i=0; i<Pos.size(); i++)	cout << "pos[" << i << "]=" << Pos[i] << endl;
	//this->UpdateFitness();
	cout << "=============================" << endl;	
	cout << " x1 : " << Pos[0] << " x2 : " << Pos[1] << " f: " << Fitness << endl;
	cout << " v1 : " << Vel[0] << " v2 : " << Vel[1] << endl;
	cout <<	" gB1: " << globalBestPos[0]<< " gB2: " << globalBestPos[1] << endl;
	cout << " in : " << inertiaParteB << " me : " << memoryParteB << " co : " << cooperationParteB;
	//cout << " pe : " << perturbation << endl << endl;
	cout << endl;
}



vector <double > particle::GetBestPos()
{
	return myBestPos;
}

void particle::SetBestPos(vector<double> newGlobalBestPos)
{
	globalBestPos=newGlobalBestPos;
}


vector <double > particle::GetPos()
{
	return Pos;
}

vector <double > particle::GetVel()
{
	return Vel;
}


void particle::Mutate(
	boost::mt19937 &rng, 
	const double &TauP)
{
	boost::normal_distribution<double> norm_dist(0.0, 1.0);
	boost::variate_generator<boost::mt19937&, boost::normal_distribution<double> >  rng_sampler(rng, norm_dist);
	// for gaussian mutation 

	double ranNum;

	ranNum=rng_sampler();
	//inertia=inertia * pow(exp(ranNum), TauP); //lognormal type 
	inertiaParteB=inertiaParteB + ranNum*TauP; // gaussian type
	if (inertiaParteB<0) inertiaParteB=0;
	if (inertiaParteB>1) inertiaParteB=1;

	ranNum=rng_sampler();
	//memory=memory * pow(exp(ranNum), TauP);
	memoryParteB = memoryParteB + ranNum*TauP;
	if (memoryParteB<0) memoryParteB = 0;
	if (memoryParteB>1) memoryParteB = 1;

	ranNum=rng_sampler();
	//cooperation=cooperation * pow(exp(ranNum), TauP);
	cooperationParteB = cooperationParteB + ranNum * TauP;
	if (cooperationParteB<0) cooperationParteB=0;
	if (cooperationParteB>1) cooperationParteB=1;


	ranNum=rng_sampler();
	//perturbation=perturbation * pow(exp(ranNum), TauP);
	perturbationParteB = perturbationParteB + ranNum*TauP;
	if (perturbationParteB<0) perturbationParteB=0;
	if (perturbationParteB>1) perturbationParteB=1;
}


void particle::Movement(
	const fvec &variaveisProblema,   
	const double &minPosMath,
	const double &maxPosMath, 
	boost::mt19937& rng, 
	const mat &W1, const mat &W2,
	const rowvec &B1, const rowvec &B2,  const rowvec &Parameter1, const rowvec &Parameter2,
	const float &actFcnFlatness, 
	const ActivationFunctions &NeuronActFunct2,
	const NormalizationMethod &NormMethod,
	const double &CpP)
{
	double inertiaTerm, memoryTerm, cooperationTerm; // perturbationTerm;
	double tempVel;
	double tempPos;
	
	boost::uniform_real<double> uniform_dist(0,1);
	boost::variate_generator<boost::mt19937&, boost::uniform_real<double> >  uniform_sampler(rng, uniform_dist);

	double ranNum;

	double disturbator;
	disturbator = uniform_sampler(); // for disturbing the global best of the particular particle
	
	unsigned int DimHidden = variaveisProblema(1);
	for (unsigned int i=0; i<DimHidden; i++)
	{
		inertiaTerm = inertiaParteB * Vel[i]; // iner * v(t-1)
		memoryTerm = memoryParteB * ( myBestPos[i] - Pos[i] ); // mem * (myBest - pos)
		
		if (uniform_sampler()<CpP) // per-dimension stochastic star - default for EPSO
		{
			ranNum = disturbator;
			globalBestPos[i]=globalBestPos[i] * (1 + perturbationParteB * ranNum);
			cooperationTerm = cooperationParteB * (globalBestPos[i] - Pos[i]); 
			// coop* (gBest - Pos)
		}
		else
		{
			cooperationTerm = 0;
		}
		tempVel = inertiaTerm + memoryTerm + cooperationTerm; // v(t)
		
		if (tempVel<minVel[i]) tempVel=minVel[i];
		if (tempVel>maxVel[i]) tempVel=maxVel[i];
		Vel[i]=tempVel; // update velocity

		tempPos=Pos[i]+Vel[i]; // x(t+1) = x(t) + v(t)

		if (tempPos<minPos[i]) 
		{
			tempPos=minPos[i];
			Vel[i]=-Vel[i]; // bounce
		}

		/*if (tempPos<capacidadeMin[i]) 
		{
			tempPos=capacidadeMin[i];
			//Vel[i]=-Vel[i]; // bounce
		}*/
		if (tempPos>maxPos[i])
		{
			tempPos=maxPos[i];
			Vel[i]=-Vel[i]; // bounce
		}
		Pos[i]=tempPos; // update position

		if (Vel[i]<minVel[i]) Vel[i]=minVel[i]; // in case of asymmetric velocity limits
		if (Vel[i]>maxVel[i]) Vel[i]=maxVel[i]; 
	}
	
	

	this->UpdateFitness(
		variaveisProblema,  
		minPosMath,
		maxPosMath, 
		W1, W2, B1, B2, Parameter1, Parameter2, actFcnFlatness, 
		NeuronActFunct2, NormMethod);

	if (ff->fitnessCompare(this->Fitness, this->myBestFitness)) //if (this->Fitness > this->myBestFitness)
	{
		this->myBestFitness = this->Fitness;
		this->myBestPos = this->Pos;
	}
}



double particle::GetSInertia()
{
	return inertiaParteB;
}

double particle::GetSMemory()
{
	return memoryParteB;
}

double particle::GetSCooperation()
{
	return cooperationParteB;
}
double particle::GetSPerturbation()
{
	return perturbationParteB;
}

