#include "swarm.h"


swarm::~swarm(void){}

//construtor do swarm do espao reduzido
swarm::swarm(
	const fvec &variaveisProblema,   
	const double &minPosMath,
	const double &maxPosMath, 
	list <HWparticle> &lastPrimeiroSwarm,
	list <HWparticle> &beforeLastPrimeiroSwarm,
	const int &numParticles, 
	const mat &W1, const mat &W2,
	const rowvec &B1, const rowvec &B2, const rowvec &Parameter1, const rowvec &Parameter2,
	float actFcnFlatness,
	const ActivationFunctions &NeuronActFunct1,
	ActivationFunctions NeuronActFunct2,
	NormalizationMethod NormMethod,
	bool newMinimize, 
	fitnessfunction &newFf, 
	boost::mt19937& rng, 
	const double &CpP, 
	const double &TauP,
	const Initi_Vel &Vel_init) : ff(newFf)
{
	this->ff = newFf; // !
	this->minimize = newMinimize;
	this->numEvalsParteB = 0;
	
	unsigned int DimIn=variaveisProblema(0);

	ff.SetMinimize(minimize);  
	
	list<HWparticle>::iterator pParticle;
	list<HWparticle>::iterator beforeParticle;
	pParticle=lastPrimeiroSwarm.begin();
	beforeParticle=beforeLastPrimeiroSwarm.begin();

	while(pParticle!=lastPrimeiroSwarm.end())
	{
		vector<double> tempPos, tempBeforePos, tempBestPos;
		for (unsigned int i=0;i<DimIn;i++)
		{
			tempPos.push_back(pParticle->HWGetPos()[i]);
			tempBestPos.push_back(pParticle->HWGetBestPos()[i]);

			tempBeforePos.push_back(beforeParticle->HWGetPos()[i]);
		}

		//Fitness=pParticle->HWGetFitness();
		inertiaParteB=(pParticle->GetInertia());
		memoryParteB=(pParticle->GetMemory());
		cooperationParteB=(pParticle->GetCooperation());
		perturbationParteB=(pParticle->GetPerturbation());

#if 0 //#if 1 reinicializa os pesos (gaussian mutation)
		boost::normal_distribution<double> norm_dist(0.0, 1.0);
		boost::variate_generator<boost::mt19937&, boost::normal_distribution<double> >  rng_sampler(rng, norm_dist);
						
		Inertia=rng_sampler();
		Memory=rng_sampler();
		Cooperation=rng_sampler();
		Perturbation=rng_sampler();
#endif
				
		EnxameParteB.push_back(particle(
			variaveisProblema,  
			minPosMath,
			maxPosMath, 
			tempPos, 
			tempBeforePos, 
			tempBestPos,
			W1, W2,	B1,	B2, Parameter1, Parameter2,
			actFcnFlatness,
			NeuronActFunct1,
			NeuronActFunct2,
			NormMethod,
			inertiaParteB, 
			memoryParteB, 
			cooperationParteB, 
			perturbationParteB,
			rng, 
			&ff, 
			CpP,
			Vel_init));
				
		pParticle++;
		beforeParticle++;
	}
	

	EnxameParteB.sort(ff.particleCompare); // may be stupid: sort list of particles according to their fitness

	this->gBestFitness = EnxameParteB.begin()->GetFitness();
	this->gBestPos = EnxameParteB.begin()->GetPos();  // get the global best for the beginning
}



unsigned int swarm::GetNumEvalsParteB()
{
	return this->numEvalsParteB;
}

double swarm::GetBestFitness()
{
	return gBestFitness;
}

vector<double> swarm::GetBestPos()
{
	return gBestPos;
}



void swarm::NextIterationSegundo(
	const fvec &variaveisProblema,   
	const double &minPosMath,
	const double &maxPosMath, 
	boost::mt19937& rng, 
	const mat &W1, const mat &W2,
	const rowvec &B1, const rowvec &B2, const rowvec &Parameter1, const rowvec &Parameter2,
	float actFcnFlatness, 
	const ActivationFunctions &NeuronActFunct2,
	const NormalizationMethod &NormMethod,
	unsigned int maxIterS, 
	const unsigned int &numberSegundo,
	const double &CpP,
	const double &TauP)
{
	particle tempParticle;

	list<particle>::iterator pParticle;
	pParticle=EnxameParteB.begin();
		
	if (ff.fitnessCompare(pParticle->GetFitness(), gBestFitness))
	{
		gBestPos = pParticle->GetPos(); // new gBest found - update.
		gBestFitness = pParticle->GetFitness(); 
	}
	
	while(pParticle!=EnxameParteB.end())
	{
		pParticle->SetBestPos(gBestPos); // new gBest set
		tempParticle=*pParticle; // replicate particle, make a copy

		tempParticle.Mutate(rng, TauP); //  mutate internal parameters
		tempParticle.Movement(
			variaveisProblema,  
			minPosMath,
			maxPosMath, 
			rng, W1, W2, B1, B2, Parameter1, Parameter2, 
			actFcnFlatness, NeuronActFunct2, NormMethod, CpP); // move the replicated particle
		
		(*pParticle).Movement(
			variaveisProblema,  
			minPosMath,
			maxPosMath, 
			rng, W1, W2, B1, B2, Parameter1, Parameter2,
			actFcnFlatness, NeuronActFunct2, NormMethod, CpP); // move the original particle

		numEvalsParteB++;
		numEvalsParteB++;

		if ( ff.particleCompare(*pParticle, tempParticle) )
		{ // original particle chosen - nothing to do
		}
		else
		{ // mutated particle chosen
			(*pParticle)=tempParticle;
		}	
		pParticle++; // iterate through all the particles
	}
	EnxameParteB.sort(ff.particleCompare); // order the list of particles

	//=======================================
	//memorization of the swarm that will be transfered to S
	// iteration finished
	//copy of the last solution for main
	if (numberSegundo==maxIterS) //nota
	{
		pParticle=EnxameParteB.begin();
		while(pParticle!=EnxameParteB.end())
		{
			tempParticle=*pParticle;
			lastSegundoSwarm.push_back(tempParticle);
			pParticle++;
		}
	}


	//=======================================
	//memorization of the swarm before the last to transfer the velocity between spaces.
	if (numberSegundo == maxIterS-1)//nota
	{
		pParticle=EnxameParteB.begin();
		while(pParticle!=EnxameParteB.end())
		{
			tempParticle=*pParticle;
			BeforeLastSegundoSwarm.push_back(tempParticle);
			pParticle++;
		}
	}

}
