//=================================
// include guard
#pragma once

//=================================
// included dependencies
#include "fitnessfunctionHW.h"
#include <armadillo>

#include <stdio.h>
#include <math.h>

#ifndef PI
#define PI 3.14159265
#endif

//=================================
// namespaces
using namespace std;
using namespace arma;


//######################################################################
// MATH FUNCTIONS
//######################################################################


// Rosenbrock
class Rosenbrock : public HWfitnessfunction
{
public:
	Rosenbrock():HWfitnessfunction(){ }; // constructor

	double calcFitness(
		const fvec &variaveisProblema,   
		vector<double> X)
	{
		
		unsigned int DimIn = variaveisProblema(0);

		double fit = 0.0;
						
		for (unsigned int i=0; i<(DimIn-1); i++) 
		{
			fit=fit + (100*pow((X[i+1]-X[i]*X[i]),2)+pow((X[i]-1),2));	
		}
		return fit;
	}
};

//-----------------------------------------------------------------------------------------------------------------------------------------
// Sphere
class Sphere : public HWfitnessfunction
{
public:
	Sphere():HWfitnessfunction(){ }; // constructor

	double calcFitness(
		const fvec &variaveisProblema,   
		vector<double> X) 
	{
		unsigned int DimIn = variaveisProblema(0);
		double fit = 0.0;


		for (unsigned int i=0; i<(DimIn); i++)
		{
			fit=fit+ (X[i] - 1.0)*(X[i] - 1.0); // sphere, sum (xi - 1)^2
		}
		return fit;
	}
};

//-----------------------------------------------------------------------------------------------------------------------------------------
// ShifSphere
class ShifSphere : public HWfitnessfunction
{
public:
	ShifSphere():HWfitnessfunction(){ }; // constructor

	double calcFitness(
		const fvec &variaveisProblema,   
		vector<double> X) 
	{
		unsigned int DimIn = variaveisProblema(0);
		double fit = 0.0;


		for (unsigned int i=0; i<(DimIn); i++)
		{
			fit=fit+ (X[i] - 1.0)*(X[i] - 1.0); // sphere, sum (xi - 1)^2
		}
		fit = fit - 450.0;
		return fit;
	}
};


//-----------------------------------------------------------------------------------------------------------------------------------------
// ShifRastrigin
class ShifRastrigin : public HWfitnessfunction
{
public:
	ShifRastrigin():HWfitnessfunction(){ }; // constructor

	double calcFitness(
		const fvec &variaveisProblema,   
		vector<double> X) 
	{
		unsigned int DimIn = variaveisProblema(0);
		
		double fit = 0.0;
		
		for (unsigned int i=0; i<(DimIn); i++)
		{
			fit=fit + (X[i]*X[i] - 10.0*cos(2.0*PI*X[i])+10.0);
		}
		fit =fit - 330.0;
		
		return fit;
	}
};

//-----------------------------------------------------------------------------------------------------------------------------------------
// ShifSchwefel
class ShifSchwefel : public HWfitnessfunction
{
public:
	ShifSchwefel():HWfitnessfunction(){ }; // constructor

	double calcFitness(
		const fvec &variaveisProblema,   
		vector<double> X) 
	{
		unsigned int DimIn = variaveisProblema(0);
		
		double fit = 0.0;
		double sumIn=0.0;

		for (unsigned int i=0; i<(DimIn); i++)
		{
			for (unsigned int j=0; j<i; j++)
			{
				sumIn=sumIn+X[j];  //!!!!!!!!!
			}
			fit = fit+sumIn*sumIn;

			sumIn=0.0;
		}
		fit =fit - 450.0;
		
		return fit;
	}
};


//--------------------------------------------------------------------------------------
class Alpine : public HWfitnessfunction
{
public:
	Alpine():HWfitnessfunction(){ }; // constructor

	double calcFitness(
		const fvec &variaveisProblema,   
		vector<double> X)
	{
		unsigned int DimIn = variaveisProblema(0);
		double fit = 1.0;
		double mult=1.0;
		
		for (unsigned int i=0; i<(DimIn); i++) 
		{
			fit=fit * sin(X[i]);
			mult = mult*X[i];
		}
		fit=fit*sqrt(mult);

		return fit;
	}
};


//-------------------------------------------------------------------------
class Schaffer : public HWfitnessfunction
{
public:
	Schaffer():HWfitnessfunction(){ }; // constructor

	double calcFitness(
		const fvec &variaveisProblema,   
		vector<double> X)
	{
		unsigned int DimIn = variaveisProblema(0);
		double fit = 0.0;

		// s usa a posio x e y, as outras no interessam... 
		double x1 = X[0];
		double x2 = X[1];

		fit = 0.5 + ((std::sin(std::sqrt(x1*x1+x2*x2)))*(std::sin(std::sqrt(x1*x1+x2*x2))) - 0.5) / ((1.0 + 0.001*(x1*x1+x2*x2))*(1.0 + 0.001*(x1*x1+x2*x2)));

		return fit;
	}
};


//---------------------------------------------------------------------------------
class Griewank : public HWfitnessfunction
{
public:
	Griewank():HWfitnessfunction(){ }; // constructor

	double calcFitness(
		const fvec &variaveisProblema,   
		vector<double> X)

	{
		unsigned int DimIn = variaveisProblema(0);
		double fit = 0.0;
		double p = 1.0;
		for (unsigned int i=0; i<(DimIn); i++)
		{  
			fit = fit + X[i]*X[i];
			p = p * (cos(X[i]/sqrt((double) i+1)));
		}
		fit = fit / 4000 - p + 1;
		return fit;
	}
};
