#include ".\particleHW.h"

//=================================
// destructor
HWparticle::~HWparticle(void){}

//=================================
// constructor, default
HWparticle::HWparticle(void){}

//=================================
// constructor for new particles of Part A
HWparticle::HWparticle(
	const fvec &variaveisProblema,  
	const double &minPosMath,
	const double &maxPosMath, 
	boost::mt19937 &rng, 
	HWfitnessfunction *pFf, 
	const double &CpG, 
	const double &TauG)
{
	
	unsigned int DimIn = variaveisProblema(0);
	for (unsigned int i=0; i<DimIn; i++)
	{
		//==============================
		//generates positions with random numbers
		boost::uniform_real<double> uniform_pos(minPosMath, maxPosMath);  
		boost::variate_generator<boost::mt19937&, boost::uniform_real<double> >  uniform_sampler_position(rng, uniform_pos);
		Pos.push_back(uniform_sampler_position());
		
		//==============================
		//generates velocities with random numbers
		boost::uniform_real<double> uniform_vel(minPosMath, maxPosMath);
		boost::variate_generator<boost::mt19937&, boost::uniform_real<double> >  uniform_sampler_velocities(rng, uniform_vel); 
		Vel.push_back(uniform_sampler_velocities());
	}
	
	//==============================
	//generates weights with random numbers (between 0 and 1)
	boost::uniform_real<double> uniform_weight(0,1);
	boost::variate_generator<boost::mt19937&, boost::uniform_real<double> >  uniform_sampler_weights(rng, uniform_weight); 

	inertia=uniform_sampler_weights();
	memory=uniform_sampler_weights();
	cooperation=uniform_sampler_weights();
	perturbation=uniform_sampler_weights(); 
	
	//==============================
	this->ff = pFf;
		
	myBestPos=Pos;
	globalBestPos=Pos;	
	
	this->HWUpdateFitness(variaveisProblema);
	myBestFitness=Fitness;
}

//=================================
// constructor for particles of Part C (position and velocity known in advance)
HWparticle::HWparticle(
	const fvec &variaveisProblema,  
	const double &minPosMath,
	const double &maxPosMath, 
	vector<double> tempPos,
	vector<double> tempBeforePos,
	vector<double> tempBestPos,
	const mat &W2, const rowvec &B2, const rowvec &Parameter1, const rowvec &Parameter2,
	const float &actFcnFlatness,
	const ActivationFunctions &NeuronActFunct2,
	const NormalizationMethod &NormMethod,
	boost::mt19937 &rng, 
	double Inertia, double Memory, double Cooperation, double Perturbation,
	HWfitnessfunction *pFf, 
	const double &CpG,
	const Initi_Vel &Vel_init)
{
	
	this->inertia=Inertia;
	this->memory=Memory;
	this->cooperation=Cooperation;
	this->perturbation=Perturbation;
	
	unsigned int DimIn = variaveisProblema(0);
	unsigned int DimHidden = variaveisProblema(1);
		
	//--------------------------------------
	//Primeiro Expandir!!!
	unsigned short numPatterns = 3;
	
	//================================
	//extracting the particles
	mat Com_Pos (numPatterns, DimHidden);

	for (unsigned int j=0; j<DimHidden; j++)
	{
		Com_Pos(0,j)=tempPos[j];
		Com_Pos(1,j)=tempBestPos[j];
		Com_Pos(2,j)=tempBeforePos[j];
	}
	
	//================================
	//expanding the particles
	mat To_expand_Pos (numPatterns, DimIn);

	General_functions::transitionLayer (
		To_expand_Pos, //para ser actualizado
		numPatterns, //os dois vetores a serem expandidos
		DimHidden, 
		DimIn, 
		Com_Pos, 
		W2, 
		B2, 
		actFcnFlatness, 
		NeuronActFunct2);
		
	//================================
	//funo de desnormalizao	
	mat DesNormPositions (numPatterns, DimIn);

	DesNormPositions =  Normalization::DES_Normalization_function (
		Parameter1, 
		Parameter2, 
		To_expand_Pos, 
		numPatterns, 
		DimIn, 
		NormMethod);


	expGlobalBestPos.resize(DimIn);
	expPos.resize(DimIn);
	expBeforePos.resize(DimIn);

	for (unsigned int j=0; j<DimIn; j++)
	{
		expPos[j]=DesNormPositions(0,j); //expanded position of particle 
		expGlobalBestPos[j]=DesNormPositions(1,j); // expanded global best position of particle
		expBeforePos[j]=DesNormPositions(2,j); // expanded position at t-1 of particle 
		

		switch (Vel_init)
		{ 
			case RANDOM_VEL: 
			{
				//velocities are initialized as random
				//define distribution for velocities according to [minVel, maxVel] for i-th dimension
				boost::uniform_real<double> uniform_vel(minPosMath, maxPosMath);
				boost::variate_generator<boost::mt19937&, boost::uniform_real<double> >  uniform_sampler_velocities(rng, uniform_vel); 
				Vel.push_back(uniform_sampler_velocities());
				break;
			}
			case TRANSFER_VEL: 
			{
				//velocities are initialize as the diference V(k) = X(k)-X(k-1) 
				this->Vel.push_back(expPos[j]-expBeforePos[j]);
				break;
			}
			default: cout << "ERROR!!! ";
		}


		
	}

	//------------------COLOCAR O PRIMEIRO ENXAME DENTRO DO DOMNIO~!!!!

	
	double maxPos = maxPosMath; double minPos =minPosMath; 
	

	for (unsigned int i=0; i<DimIn; i++)
	{
	
		//os limites do domnio funcionam como um espelho!!!! ou apenas coloco inrcia a zero

		if (expPos[i]<minPos)
		{
			expPos[i]=minPos;
		}
		//Vel[i]=-Vel[i];}
		//inertia=0.0;}

		if (expPos[i]>maxPos)
		{
			expPos[i]=maxPos;
		}
		//Vel[i]=-Vel[i];}
		//inertia=0.0;}
	}

	for (unsigned int i=0; i<DimIn; i++)
    {
        this->Pos.push_back(expPos[i]);
        this->myBestPos.push_back(expPos[i]);
		this->globalBestPos.push_back(expGlobalBestPos[i]);
    }

	this->Vel = Vel;
	this->ff = pFf;
		
	this->HWUpdateFitness(variaveisProblema);
	myBestFitness=Fitness;
}

//=================================
// gets the current fitness value but does not recalculate fitness each time
double HWparticle::HWGetFitness()
{
	return Fitness;
}

//=================================
// update the fitness
void HWparticle::HWUpdateFitness(const fvec &variaveisProblema)
{
	this->Fitness=this->ff->calcFitness(variaveisProblema, Pos);
}

//=================================
// print function
void HWparticle::HWPrint()
{
	cout << "=============================" << endl;	
	cout << " x1 : " << Pos[0] << " x2 : " << Pos[1] << " f: " << Fitness << endl;
	cout << " v1 : " << Vel[0] << " v2 : " << Vel[1] << endl;
	cout <<	" gB1: " << globalBestPos[0]<< " gB2: " << globalBestPos[1] << endl;
	cout << " in : " << inertia << " me : " << memory << " co : " << cooperation;
	//cout << " pe : " << perturbation << endl << endl;
	cout << endl;
}

//=================================
//get the particle's own previous best position (pBest)
vector <double > HWparticle::HWGetBestPos()
{
	return myBestPos;
}

//=================================
//set global best position gBest for this particle
void HWparticle::HWSetBestPos(vector<double> newGlobalBestPos)
{
	globalBestPos=newGlobalBestPos;
}

//=================================
//get particle's current position (location, vector X)
vector <double > HWparticle::HWGetPos()
{
	return Pos;
}

//=================================
//get particle's current velocity
vector <double > HWparticle::HWGetVel()
{
	return Vel;
}

//=================================
// mutate the internal parameters (EPSO)
//rng is parameter so that lognormal mutation can be called
void HWparticle::HWMutate(
	boost::mt19937 &rng, 
	const double &TauG)
{
	boost::normal_distribution<double> norm_dist(0.0, 1.0);
	boost::variate_generator<boost::mt19937&, boost::normal_distribution<double> >  rng_sampler(rng, norm_dist);
	
	double ranNum;
	ranNum=rng_sampler();
	//do {ranNum=rng_sampler();}
	//while (TauG*ranNum<-0.95);

	//inertia=inertia * pow(exp(ranNum), TauG); //lognormal type 
#if 1
	inertia=inertia + ranNum*TauG; // gaussian type
	if (inertia<0) inertia=0;
	if (inertia>1) inertia=1;

	ranNum=rng_sampler();
	//memory=memory * pow(exp(ranNum), TauG);
	memory=memory + ranNum*TauG;
	if (memory<0) memory=0;
	if (memory>1) memory=1;

	ranNum=rng_sampler();
	//cooperation=cooperation * pow(exp(ranNum), TauG);
	cooperation=cooperation + ranNum * TauG;
	if (cooperation<0) cooperation=0;
	if (cooperation>1) cooperation=1;
	
	ranNum=rng_sampler();
	//perturbation=perturbation * pow(exp(ranNum), TauG);
	perturbation=perturbation + ranNum*TauG;
	if (perturbation<0) perturbation=0;
	if (perturbation>1) perturbation=1;
#endif

#if 0 // w=w[1+tauN(0,1)] version
	inertia=inertia + inertia*ranNum*TauG; 
	//if (inertia<0) inertia=0;
	//if (inertia>1) inertia=10;

	ranNum=rng_sampler();
	//memory=memory * pow(exp(ranNum), TauG);
	memory=memory + memory*ranNum*TauG;
	//if (memory<0) memory=0;
	//if (memory>1) memory=10;

	ranNum=rng_sampler();
	//cooperation=cooperation * pow(exp(ranNum), TauG);
	cooperation=cooperation + cooperation*ranNum * TauG;
	//if (cooperation<0) cooperation=0;
	//if (cooperation>1) cooperation=5;
	
	ranNum=rng_sampler();
	//perturbation=perturbation * pow(exp(ranNum), TauG);
	perturbation=perturbation + perturbation*ranNum*TauG;
	//if (perturbation<0) perturbation=0;
	//if (perturbation>1) perturbation=5;
#endif
}

//=================================
//mutao que consiste em reinicializar os pesos das particulas de forma aleatoria
void HWparticle::RandomHWMutate(boost::mt19937 &rng)
{
   
    //define distribution for weights
    boost::uniform_real<double> uniform_weight(0,5);
    boost::variate_generator<boost::mt19937&, boost::uniform_real<double> >  uniform_sampler_weights(rng, uniform_weight);

    inertia=uniform_sampler_weights();
    memory=uniform_sampler_weights();
    cooperation=uniform_sampler_weights();
    perturbation=uniform_sampler_weights();
}

//=================================
// perform the movement of particles according to EPSO rules
void HWparticle::Movement(
	boost::mt19937& rng,
	const fvec &variaveisProblema,  
	const double &minPosMath,
	const double &maxPosMath, 
	const double &CpG,
	const Initi_Vel &Vel_init,
	const unsigned int &number, //numero da iteraao relativa a parte A e parte C
	const unsigned int &maxIterI, 
	const unsigned int &maxIterS)
{
	double inertiaTerm, memoryTerm, cooperationTerm; // perturbationTerm;
	double tempVel;
	double tempPos;
		
	boost::uniform_real<double> uniform_dist(0,1);
	boost::variate_generator<boost::mt19937&, boost::uniform_real<double> >  uniform_sampler(rng, uniform_dist);

	double ranNum;
	double disturbator;
	disturbator = uniform_sampler(); // for disturbing the global best of the particular particle
	
	
	unsigned int DimIn = variaveisProblema(0);
	
	
	double maxPos=maxPosMath, minPos=minPosMath; 
	
	for (unsigned int i=0; i<DimIn; i++)
	{
		inertiaTerm = inertia * Vel[i]; // iner * v(t-1)
		memoryTerm = memory * ( myBestPos[i] - Pos[i] ); // mem * (myBest - pos)
			
		if (uniform_sampler()<CpG) // per-dimension stochastic star - default for EPSO
		{
			ranNum = disturbator;
			globalBestPos[i]=globalBestPos[i] * (1 + perturbation * ranNum);
			cooperationTerm = cooperation * (globalBestPos[i] - Pos[i]);
		}
		
		else
		{	
			cooperationTerm = 0;
		}

		if((number == maxIterI+1) || (number == (maxIterI+maxIterS+1))) //nota
		{
			switch (Vel_init)
			{ 
				case RANDOM_VEL:
				{
					//induces random velocities in two moments of the EPSO experiment: 
					//in maxIterI and in (maxIterI+maxIterS) --> corresponds to the moments where Hybrid transits from S to S', and S' to S, respectively
					boost::uniform_real<double> uniform_vel(minPosMath, maxPosMath);
					boost::variate_generator<boost::mt19937&, boost::uniform_real<double> >  uniform_sampler_velocities(rng, uniform_vel); 
					tempVel = uniform_sampler_velocities();	
					break;
				}
				case TRANSFER_VEL: 
				{
					//by default the EPSO evolves normals, including the evolution of the velocities	
					//normal  v(t)
					tempVel = inertiaTerm + memoryTerm + cooperationTerm; 
					break;
				}
				default: cout << "ERROR!!! ";
			}
		}
		else
		{
			tempVel = inertiaTerm + memoryTerm + cooperationTerm; 
		}
		

		if (tempVel<minPosMath) {tempVel=minPosMath;}
		if (tempVel>maxPosMath) {tempVel=maxPosMath;}

		Vel[i]=tempVel; // update velocity

		tempPos=Pos[i]+Vel[i]; // x(t+1) = x(t) + v(t)
			
		
		//=================================
		//os limites do domnio funcionam como um espelho!!!! ou apenas coloco inrcia a zero

		if (tempPos<minPos)
		{
			tempPos=minPos;
			Vel[i]=-Vel[i];
		}
		//inertia=0.0;}

		if (tempPos>maxPos)
		{
			tempPos=maxPos;
			Vel[i]=-Vel[i];
		}
		//inertia=0.0;}

		//a velocidade da particula ser mais pastosa qd sair dos limites
#if 0
		if (tempPos<minPos[0])
		{tempPos=minPos[0];
		Vel[i]=0.1*Vel[i];}

		if (tempPos>maxPos[0])
		{tempPos=maxPos[0];
		Vel[i]=0.1*Vel[i];}
#endif
		
		//=================================
		// update position
		Pos[i]=tempPos; 
		
		//=================================
		//ensures velocities limits
		if (Vel[i]<minPosMath) {Vel[i]=minPosMath;} // in case of asymmetric velocity limits
		if (Vel[i]>maxPosMath) {Vel[i]=maxPosMath;}
	}



	this->HWUpdateFitness(variaveisProblema);

	if (ff->HWfitnessCompare(this->Fitness, this->myBestFitness)) //if (this->Fitness > this->myBestFitness)
	{	
		this->myBestFitness = this->Fitness;
		this->myBestPos = this->Pos;
	}
}


//=================================
//get epso parameters
double HWparticle::GetInertia()
{
	return inertia;
}

double HWparticle::GetMemory()
{
	return memory;
}

double HWparticle::GetCooperation()
{
	return cooperation;
}

double HWparticle::GetPerturbation()
{
	return perturbation;
}


