//=================================
// include guard
#pragma once

//=================================
// forward declared dependencies
class HWfitnessfunction;

//=================================
// included dependencies
#include <boost/random.hpp>
#include <boost/random/uniform_real.hpp>
#include <boost/random/lognormal_distribution.hpp>

#include <cassert>
#include <cstdlib>
#include <algorithm>
#include <ctime>
#include <fstream>
#include <vector>
#include <iostream>
#include <armadillo>
#include <list>

#include "fitnessfunctionHW.h"

#include "..\ITL_networks\General_Functions.h"
#include "..\ITL_networks\Normalization_functions.h"

#include "..\enumerations.h"

//=================================
// namespaces
using namespace std;
using namespace arma;

//=================================
// the HWparticle class
class HWparticle
{
public:
	//=================================
	// destructor
	~HWparticle(void); 

	//=================================
	// constructor, default
	HWparticle(void); 
	
	//=================================
	// constructor for new particles of Part A
	HWparticle(
		const fvec &variaveisProblema,
		const double &minPosMath,
		const double &maxPosMath, 
		boost::mt19937& rng, 
		HWfitnessfunction *pFf, 
		const double &CpG,
		const double &TauG);
	
	//=================================
	// constructor for particles of Part C (position and velocity known in advance)
	HWparticle(
		const fvec &variaveisProblema,  
		const double &minPosMath,
		const double &maxPosMath,
		vector<double> tempPos,
		vector<double> tempBeforePos,
		vector<double> tempBestPos,
		const arma::mat &W2,
		const arma::rowvec &B2,  const rowvec &Parameter1, const rowvec &Parameter2,
		const float &actFcnFlatness,
		const ActivationFunctions &NeuronActFunct2,
		const NormalizationMethod &NormMethod,
		boost::mt19937 &rng, 
		double Inertia,
		double Memory,
		double Cooperation,
		double Perturbation,
		HWfitnessfunction *pFf, 
		const double &CpG,
		const Initi_Vel &Vel_init);
		

	//=================================
	// gets the current fitness value but does not recalculate fitness each time
	double HWGetFitness(); 
	

	//=================================
	// update the fitness
	void HWUpdateFitness(const fvec &variaveisProblema); 
	

	//=================================
	// print function
	void HWPrint(); 

	//=================================
	//get the particle's own previous best position (pBest)
	vector <double > HWGetBestPos(); 
	
	//=================================
	//set global best position gBest for this particle
	void HWSetBestPos(std::vector<double > newGlobalBestPos); 
	
	//=================================
	//get particle's current position (location, vector X)
	vector <double > HWGetPos(); 
	
	//=================================
	//get particle's current velocity
	vector <double > HWGetVel(); 

	//=================================
	// mutate the internal parameters (EPSO)
	//rng is parameter so that lognormal mutation can be called
	void HWMutate(
		boost::mt19937& rng, 
		const double &TauG); 

	//=================================
	//mutao que consiste em reinicializar os pesos das particulas de forma aleatoria
	void RandomHWMutate(boost::mt19937 &rng); 
											
	//=================================
	// perform the movement of particles according to EPSO rules
	void Movement(
		boost::mt19937& rng, 
		const fvec &variaveisProblema,   
		const double &minPosMath,
		const double &maxPosMath, 
		const double &CpG,
		const Initi_Vel &Vel_init,
		const unsigned int &number, //numero da iteraao relativa a parte A e parte C
		const unsigned int &maxIterI, 
		const unsigned int &maxIterS); 
		

	//=================================
	//get epso parameters
	double GetInertia();
	double GetMemory();
	double GetCooperation();
	double GetPerturbation();
		
	
private:
	double Fitness; // stores current fitness //ok
	vector<double> Pos; // position
	vector<double> Vel; // velocity
	vector<double> myBestPos; // particle previous best, pBest
	vector<double> globalBestPos; //global best, gBest
	double myBestFitness; //particle's best fitness in myBestPos (pBest)

	double inertia;
	double memory;
	double cooperation;
	double perturbation; 
	
	//=================================
	// pointer to fitness function
	HWfitnessfunction *ff; 

	//=================================
	vector<double> maxVel;
	vector<double> minVel;
	
	vector <double> expPos, expGlobalBestPos, expBeforePos;
	
};
