#include "swarmHW.h"


//=================================
// destructor
HWswarm::~HWswarm(void){}


//=================================
HWswarm::HWswarm(
	boost::mt19937 &rng, //ok
	const unsigned int &numParticles, //ok
	const fvec &variaveisProblema,  
	const double &minPosMath,
	const double &maxPosMath, 
	bool newMinimize, 
	HWfitnessfunction &newFf, 
	const double &CpG,  //ok
	const double &TauG) : ff(newFf) //ok
{
	this->numEvalsParteA = 0;
	this->ff = newFf; // !
	this->minimize = newMinimize;

	ff.HWSetMinimize(minimize);

	for (unsigned int i=0; i<numParticles; i++) // initialize list of particles
	{
		EnxameParteA.push_back(
			HWparticle(
				variaveisProblema,  
				minPosMath,
				maxPosMath,
				rng, 
				&ff, 
				CpG, 
				TauG));
	}

	
	EnxameParteA.sort(ff.particleCompare); // may be stupid: sort list of particles according to their fitness

	this->gBestFitness = EnxameParteA.begin()->HWGetFitness();
	this->gBestPos = EnxameParteA.begin()->HWGetPos();  // get the global best for the beginning

}




HWswarm::HWswarm(
	boost::mt19937 &rng,
	list <particle> &lastSegundoSwarm,
	list <particle> &beforeLastSegundoSwarm,
	const unsigned int &numParticles,
	const fvec &variaveisProblema,
	const double &minPosMath,
	const double &maxPosMath, 
	const mat &W2, const rowvec &B2, 
	const rowvec &Parameter1, const rowvec &Parameter2,
	const float &actFcnFlatness,
	const ActivationFunctions &NeuronActFunct2,
	const NormalizationMethod &NormMethod,
	bool newMinimize, 
	HWfitnessfunction &newFf, 
	const double &CpG, 
	const double &TauG,
	const Initi_Vel &Vel_init):ff(newFf)
{
	this->numEvalsParteC = 0;
	this->minimize = newMinimize;
	this->ff = newFf;	
	
	unsigned int DimHidden = variaveisProblema (1);
	ff.HWSetMinimize(minimize);  
	
	//mximos e minimos para normalizar
	list<particle>::iterator pParticle;
	list<particle>::iterator beforeParticle;
	pParticle      = lastSegundoSwarm.begin();
	beforeParticle = beforeLastSegundoSwarm.begin();
	

	while(pParticle!=lastSegundoSwarm.end())
	{
		vector<double> tempPos,/* tempVel, */tempBestPos, tempBeforePos;
		for (unsigned int i=0; i<DimHidden; i++)
		{
			tempPos.push_back(pParticle->GetPos()[i]);
			//tempVel.push_back(pParticle->GetVel()[i]);
			tempBestPos.push_back(pParticle->GetBestPos()[i]);
			
			tempBeforePos.push_back(beforeParticle->GetPos()[i]);
		}

		Fitness=pParticle->GetFitness();

		Inertia=pParticle->GetSInertia();
		Memory=pParticle->GetSMemory();
		Cooperation=pParticle->GetSCooperation();
		Perturbation=pParticle->GetSPerturbation();

		EnxameParteC.push_back(HWparticle(
			variaveisProblema,  
			minPosMath, maxPosMath, 
			tempPos,
			tempBeforePos, 
			tempBestPos,
			W2, B2, Parameter1, Parameter2,
			actFcnFlatness,
			NeuronActFunct2,
			NormMethod,
			rng,
			Inertia,
			Memory, 
			Cooperation, 
			Perturbation, 
			&ff, 
			CpG,
			Vel_init));

		pParticle++;
		beforeParticle++;
	}
		
	EnxameParteC.sort(ff.particleCompare); // may be stupid: sort list of particles according to their fitness

	this->gBestFitness = EnxameParteC.begin()->HWGetFitness();
	this->gBestPos = EnxameParteC.begin()->HWGetPos();  // get the global best for the beginning
}


void HWswarm::NextIterationPrimeiro(
	boost::mt19937 &rng, 
	const unsigned int &numberPrimeiro, //numero da iteraao relativa a parte A
	const unsigned int &maxIterI, 
	const unsigned int &maxIterS, 
	//const unsigned int &bigMaxIterI,
	const fvec &variaveisProblema,   
	const double &minPosMath,
	const double &maxPosMath, 
	const unsigned int &trainDim, //ok 
	const unsigned int &TBegin, //ok 
	const unsigned int &TEnd,//ok 
	const unsigned int &testDim,//ok 
	const double &CpG, //ok
	const double &TauG,
	const Initi_Vel &Vel_init) //ok
{
	
	HWparticle tempParticle;

	list<HWparticle>::iterator pParticle;
	pParticle=EnxameParteA.begin();	// this is the best particle

	if (ff.HWfitnessCompare(pParticle->HWGetFitness(), gBestFitness))
	{
		gBestPos = pParticle->HWGetPos(); // new gBest found - update.
		gBestFitness = pParticle->HWGetFitness(); 
	}
	//se o bloco #if seguinte estiner activo, faz a reinicializao dos pesos
	

	while(pParticle!=EnxameParteA.end())
	{
		pParticle->HWSetBestPos(gBestPos); //new gBest set
		tempParticle=*pParticle; // replicate particle, make a copy
		tempParticle.HWMutate(rng, TauG); //  tau = 0.2, mutate internal parameters

		tempParticle.Movement(
			rng, 
			variaveisProblema, 
			minPosMath, maxPosMath, 
			CpG, 
			Vel_init,
			numberPrimeiro,
			maxIterI,
			maxIterS); // move the replicated particle

		(*pParticle).Movement(
			rng, 
			variaveisProblema, 
			minPosMath, maxPosMath,
			CpG, 
			Vel_init,
			numberPrimeiro,
			maxIterI,
			maxIterS); // move the original particle

		numEvalsParteA++;
		numEvalsParteA++;

		if ( ff.particleCompare(*pParticle, tempParticle) )
		{ // original particle chosen - nothing to do
		}
		else
		{ // mutated particle chosen
			(*pParticle)=tempParticle;
		}	
		pParticle++; // iterate through all the particles
	}

	EnxameParteA.sort(ff.particleCompare); // order the list of particles
	
	if (numberPrimeiro==(maxIterI)) //nota
	{
		pParticle=EnxameParteA.begin();
		while(pParticle!=EnxameParteA.end())
		{
			lastPrimeiroSwarm.push_back(*pParticle);
			pParticle++;
		}
	}

	//======================================
	//to store the before Pos to transfer velocity from S to S'
	if (numberPrimeiro==(maxIterI-1))//nota
	{
		pParticle=EnxameParteA.begin();
		while(pParticle!=EnxameParteA.end())
		{
			beforeLastPrimeiroSwarm.push_back(*pParticle);
			pParticle++;
		}
	}
	// iteration finished
	
}


void HWswarm::NextIterationTerceiro(
	boost::mt19937& rng, //ok
	const unsigned int &numberTerceiro, //ok
	const unsigned int &maxIterF, //maxIter final
	const fvec &variaveisProblema,  
	const double &minPosMath,
	const double &maxPosMath, 
	const double &CpG,//ok
	const double &TauG,
	const Initi_Vel &Vel_init)//ok
{
	HWparticle tempParticle;  

	list<HWparticle>::iterator pParticle;
	pParticle=EnxameParteC.begin();	// this is the best particle

	if (ff.HWfitnessCompare(pParticle->HWGetFitness(), gBestFitness))
	{
		gBestPos = pParticle->HWGetPos(); // new gBest found - update.
		gBestFitness = pParticle->HWGetFitness(); 
	}


	while(pParticle!=EnxameParteC.end())
	{
		pParticle->HWSetBestPos(gBestPos); // new gBest set
		tempParticle=*pParticle; // replicate particle, make a copy
		tempParticle.HWMutate(rng, TauG); //  tau = 0.2, mutate internal parameters

		tempParticle.Movement(
			rng, 
			variaveisProblema, 
			minPosMath, 
			maxPosMath, 
			CpG, 
			Vel_init,
			numberTerceiro,
			maxIterF+1000,
			maxIterF+1000); // move the replicated particle

		(*pParticle).Movement(
			rng, 
			variaveisProblema, 
			minPosMath, maxPosMath, 
			CpG, 
			Vel_init,
			numberTerceiro,
			maxIterF+1000,
			maxIterF+1000); // move the original particle

		numEvalsParteC++;
		numEvalsParteC++;

		if ( ff.particleCompare(*pParticle, tempParticle) )
		{ // original particle chosen - nothing to do
		}
		else
		{ // mutated particle chosen
			(*pParticle)=tempParticle;
		}	
		pParticle++; // iterate through all the particles
	}

	EnxameParteC.sort(ff.particleCompare); // order the list of particles
}


unsigned int HWswarm::HWGetNumEvalsParteA()
{
	return this->numEvalsParteA;
}

unsigned int HWswarm::HWGetNumEvalsParteC()
{
	return this->numEvalsParteC;
}


double HWswarm::HWGetBestFitness()
{
	return gBestFitness;
}

vector<double> HWswarm::HWGetBestPos()
{
	return gBestPos;
}
