//================================
// include guard
#pragma once

//=================================
// included dependencies
#include <iostream>
#include <fstream>
#include <iomanip>
#include <ctime>
#include <list>
#include <vector>
#include <armadillo>

#include <boost/random.hpp>

#include "particleHW.h"
#include "fitnessfunctionHW.h"

#include "../EPSO/swarm.h"
#include "../EPSO/particle.h"
#include "..\enumerations.h"

//=================================
// namespaces
using namespace std;
using namespace arma;

//=================================
// the HWswarm class
class HWswarm
{
public:
	//=================================
	// destructor
	~HWswarm(void); 

	//=================================
	HWswarm(
		boost::mt19937& rng, 
		const unsigned int &numParticles, 
		const fvec &variaveisProblema,  
		const double &minPosMath,
		const double &maxPosMath, 
		bool newMinimize,
		HWfitnessfunction &ff, 
		const double &CpG, 
		const double &TauG); // constructor
	
	//=================================
	//constructor for the last iterations after the search in the reduced space
	HWswarm(
		boost::mt19937 &rng,
		list<particle> &lastSegundoSwarm,
		list <particle> &beforeLastSegundoSwarm,
		const unsigned int &numParticles, 
		const fvec &variaveisProblema,   
		const double &minPosMath,
		const double &maxPosMath, 
		const mat &W2,
		const rowvec &B2, const rowvec &Parameter1, const rowvec &Parameter2,
		const float &actFcnFlatness,
		const ActivationFunctions &NeuronActFunct2,
		const NormalizationMethod &NormMethod,
		bool minimize, 
		HWfitnessfunction &ff, 
		const double &CpG, 
		const double &TauG,
		const Initi_Vel &Vel_init);
	
	//=================================
	void NextIterationPrimeiro(
		boost::mt19937& rng,
		const unsigned int &numberPrimeiro, 
		const unsigned int &maxIterI, 
		const unsigned int &maxIterS, 
	//	const unsigned int &bigMaxIterI,
		const fvec &variaveisProblema,   
		const double &minPosMath,
		const double &maxPosMath, 
		const unsigned int &trainDim, //ok 
		const unsigned int &TBegin, //ok 
		const unsigned int &TEnd,//ok 
		const unsigned int &testDim,//ok 
		const double &CpG,
		const double &TauG,
		const Initi_Vel &Vel_init);

	//=================================
	void NextIterationTerceiro(
		boost::mt19937& rng,
		const unsigned int &numberTerceiro, //ok
		const unsigned int &maxIterF, //maxIter final
		const fvec &variaveisProblema,   
		const double &minPosMath,
		const double &maxPosMath, 
		const double &CpG,
		const double &TauG,
		const Initi_Vel &Vel_init);
	


	unsigned int HWGetNumEvalsParteA();
	unsigned int HWGetNumEvalsParteC();

	double HWGetBestFitness();
	vector<double> HWGetBestPos();
	

	// note: if designed, swarm's copy constructor needs to copy the state of RNG generator!
	
	//vector <double> GetI();
	//vector <double> GetM();
	//vector <double> GetC();
	//vector <double> GetP();

	list<HWparticle> EnxameParteA;	
	list<HWparticle> EnxameParteC;
	
	list<HWparticle> lastPrimeiroSwarm;//list holding the last swarm for the neural net
	list<HWparticle> lastTerceiroSwarm;

	//======================================
	//to transfer velocity to reduced space as V(k) = X(k) - X(k-1)
	list<HWparticle> beforeLastPrimeiroSwarm; //list holding the particles of the moment before the last Primeiro swarm // only to transfer velocities from S to S'

private:

	double gBestFitness; // global best's fitness
	vector<double > gBestPos; // global best position
	
	unsigned int numEvalsParteA; // number of fitness function evaluations performed
	unsigned int numEvalsParteC;
	HWfitnessfunction &ff;
	
	bool minimize; // minimizing or maximizing
		
	//---------------	para listar os pesos da melhor particula!!!!--------
	//double inertia, memory, cooperation, perturbation;
	//vector <double> I, M, C, P;
		
	double Fitness;//ok

	double Inertia;//ok
	double Memory;//ok
	double Cooperation;//ok
	double Perturbation;//ok
};
