//=================================
// include guard
#pragma once

//=================================
// included dependencies
#include <iostream>
#include <math.h>
#include <stdio.h>
#include <vector>
#include <stdlib.h>
#include <armadillo>

#include "General_Functions.h"
#include "..\enumerations.h"
//=================================
// namespaces
using namespace std;
using namespace arma;


//=================================
double Vi_function(
	const mat &TrainSet, 
	const rowvec &x, 
	const unsigned short &DimIn, 
	const double &sig2, 
	const unsigned short &TrainDim) //euclidean Distance
{
	double V_i;
	double V_ij = 0.0;
	double aux2 = 0.0;
	double dTrainDim = static_cast<double>(TrainDim);
	rowvec x_k(DimIn), dij(DimIn);

	//Calculating V_i
	for (unsigned short k = 0; k<TrainDim; k++ )
	{
		x_k = TrainSet(k,span());//x fixo, x_k vai variando
		dij = x - x_k;
		
		V_ij = General_functions::Vij_function(dij, DimIn, sig2);

		aux2 += V_ij;

		V_ij = 0.0;
	}
	V_i = aux2 / dTrainDim;
	return (V_i);// V_i = (1/patnum) * aux2;
}

//=================================
double V_function(
	const mat &TrainSet, 
	const unsigned short &DimIn, 
	const double &sig2, 
	const unsigned short &TrainDim) //Euclidean Distance
{
	//Calculating V
	double V;
	double V_i = 0.0;
	double aux3 = 0.0;
	double dTrainDim = static_cast<double>(TrainDim);
	rowvec xx(DimIn);

	for (unsigned short l=0; l<TrainDim; l++)
	{
		xx = TrainSet(l,span()); //pattern l
		V_i = Vi_function(TrainSet, xx, DimIn, sig2, TrainDim);

		aux3 += V_i;
			
		V_i = 0.0;
	}
	V = aux3 / (dTrainDim);
	return (V); //V = (1/TrainDim) * aux3;
}

//=================================
mat Cij_function (
	const unsigned short &TrainDim, 
	const unsigned short &DimIn, 
	const double &sig2, 
	const mat &TrainSet)
{
	//##### Cij Calculation #####
	// CALCULATE c_ij for input patterns s and t
	mat Cij(TrainDim, TrainDim);

	double V, Vi, Vj, Vij;
	rowvec xi(DimIn), xj(DimIn), dij(DimIn);

	V = V_function(TrainSet, DimIn, sig2, TrainDim);

	for(unsigned short i=0; i<TrainDim; i++)
	{
		xj = TrainSet(i,span()); //input of pattern t
		Vj = Vi_function(TrainSet, xj, DimIn, sig2, TrainDim);

		for (unsigned short j=0; j<TrainDim; j++)
		{
			xi = TrainSet(j,span());//input of pattern s
			dij = xi - xj;
			
			Vij = General_functions::Vij_function(dij, DimIn, sig2);
			Vi = Vi_function(TrainSet, xi, DimIn, sig2, TrainDim);

			Cij(i,j) = Vij - Vi - Vj + V;
			//cout << "i:  "<< i <<"   Cij(i,j): " << Cij(i,j) << endl;
		}
	}
	//cout << "Cij" << Cij << endl;
	//##### End of Cij Calculation #####

	return (Cij);
}

//=================================
void ED_Weights1Bias1_function (
	const unsigned short &DimHidden, 
	const unsigned short &DimIn, 
	const unsigned short &TrainDim, 
	const mat &HiddenTrain, 
	const mat &TrainSet, 
	const mat &Cij, 
	const float &actFcnFlatness, 
	const double &sig2,
	const float &momRate, 
	const float &lrCoeff1, 
	mat &Weights1, //para ser atualizada diretamente
	const mat &prev_Weights1, 
	rowvec &Bias1,  //para ser atualizada diretamente
	const rowvec &prevBias1, 
	const ActivationFunctions &NeuronActFunct)
{
	//##### BackForward Step == MaxMI Criterium #####
	rowvec y_i(DimHidden), y_j(DimHidden), dij2(DimHidden);
	double Vij_hid;
	mat delta_weights1(DimIn,DimHidden);
	
	double dTrainDim = static_cast<double>(TrainDim);

	for (unsigned short i=0; i<TrainDim; i++) //Cross samples: sample i
	{
		for (unsigned short j=0; j<TrainDim; j++) //Cross samples: sample j
		{
			y_i = HiddenTrain(i,span());
			y_j = HiddenTrain(j,span());
			dij2 = y_i - y_j;
			Vij_hid = General_functions::Vij_function(dij2, DimHidden, sig2); // Vij = G(yi-yj, 2sig2)
			
			// dV/Dw = dV/dy * dy/dw
			for(unsigned short hid=0; hid<DimHidden; hid++) //for each output neuron: derivative
			{
				// Act Function Derivative @ Y_s
				double derv_i = General_functions::DervActFcn(y_i(hid), actFcnFlatness, NeuronActFunct);

				//dV/dy = dij[k] * Vij_out * (-1.0/(2.0*sig2*TrainDimIn*TrainDimIn))
				double error_i = dij2(hid) * Cij(i,j) * Vij_hid * (-1.0/(2.0*sig2*dTrainDim*dTrainDim))*derv_i;
								
				//dy/dw = derv_ActFn_t * x_t - derv_ActFn_s * x_s
				for (unsigned short in=0; in<DimIn; in++)
				{
					double momterm = momRate * ( Weights1(in,hid) - prev_Weights1(in,hid)); //momentum term
					delta_weights1(in,hid) = ( (1.0) * lrCoeff1 * (error_i * TrainSet(i,in)) ) + momterm ; 
					Weights1(in,hid) += delta_weights1(in,hid);
				}
				
				Bias1(hid) += ((1.0) * lrCoeff1 * (error_i)) + (momRate * (Bias1(hid) - prevBias1(hid)));
			}
		}
	}
	//##### End of BackForward Step == MaxMI Criterium #####
}

//=================================
void CostFunction_ED (
	double &CostFn,  //para ser atualizada diretamente
	const unsigned short &DimHidden, 
	const unsigned short &NumPatterns, 
	const mat &HiddenMatrix, 
	const double &sig2, 
	const mat &Cij)
{
	double aux_CostFn = 0.0;
	
	double dNumPatterns = static_cast<double>(NumPatterns);

	rowvec dij3(DimHidden);

	for (unsigned short i=0; i<NumPatterns; i++)
	{
		for (unsigned short j=0; j<NumPatterns; j++)
		{
			for (unsigned short hid=0; hid<DimHidden; hid++)
			{
				dij3(hid) = HiddenMatrix(i,hid) - HiddenMatrix(j,hid);
			}
			aux_CostFn += Cij(i,j) * General_functions::Vij_function(dij3, DimHidden, sig2);
		}
	}
	
	CostFn = ( 1.0 / (dNumPatterns*dNumPatterns) ) * aux_CostFn;
	
}


