#include "Normalization_functions.h"


//=================================
rowvec Normalization::Parameter1_function (
	const mat &TrainSet,  
	const NormalizationMethod &NormMethod, 
	const unsigned short &DimIn)
{
	rowvec Parameter1 (1);

	switch (NormMethod)
	{
		case ENTRANCE_MIN_MAX: 
		{
			//Parameter1 = MaxTrain
			Parameter1.resize(DimIn);
			Parameter1= max(TrainSet,0);
			break;
		}
		
		case GLOBAL_MIN_MAX:
		{
			//Parameter1 = MaxTrain
			Parameter1(0) = max(max(TrainSet,0));
			break;
		}

		case ENTRANCE_ZSCORE:
		{
			//Parameter1 = MeanTrain
			Parameter1.resize(DimIn);
			Parameter1= mean(TrainSet,0);
			break;
		}

		case GLOBAL_ZSCORE:
		{
			//Parameter1 = MeanTrain
			Parameter1(0) = mean(mean(TrainSet,0));
			break;
		}

		case NONE:
		{
			//Parameter1 is not necessary, so it is zeros
			Parameter1.zeros();
			break;
		}
			
		default:
		{
			cout << "ERROR!!! ";
			break;
		}
	}
	 
	 return (Parameter1);
}

//=================================
rowvec Normalization::Parameter2_function (
	const mat &TrainSet,  
	const NormalizationMethod &NormMethod, 
	const unsigned short &DimIn)
{
	rowvec Parameter2 (1);

	switch (NormMethod)
	{
		case ENTRANCE_MIN_MAX: 
		{
			//Parameter2 = MinTrain;
			Parameter2.resize(DimIn);
			Parameter2= min(TrainSet,0);
			break;
		}
		
		case GLOBAL_MIN_MAX:
		{
			//Parameter2 = MinTrain;
			Parameter2(0) = min(min(TrainSet,0));
			break;
		}

		case ENTRANCE_ZSCORE:
		{
			//Parameter2 = SDev;
			Parameter2.resize(DimIn);
			Parameter2= stddev(TrainSet,0);
			break;
		}

		case GLOBAL_ZSCORE:
		{
			unsigned short vecDim = TrainSet.size();
			unsigned short linhas = vecDim /DimIn;
			vec auxParameter2 (vecDim);
			for (unsigned short in=0 ; in<linhas; in++)
			{
				for (unsigned short j=0 ; j<DimIn; j++)
				{
					auxParameter2(in*DimIn+j) = TrainSet(in, j);
				}
			}
			//Parameter2 = SDev;
			Parameter2(0) = stddev(auxParameter2);
			break;
		}

		case NONE:
		{
			//Parameter2 is not necessary, so it is zeros
			Parameter2.zeros();
			break;
		}
			
		default:
		{
			cout << "ERROR!!! ";
			break;
		}
	}
	return (Parameter2);
}

//=================================
mat Normalization::EntranceNormalizationMinMax (
	const unsigned short &linhas, 
	const unsigned short &colunas, 
	const mat &notNormalized, 
	const rowvec &MaxTrain, 
	const rowvec &MinTrain)
{
	mat EntranceNormalization(linhas,colunas);

	for(unsigned short te=0; te<linhas; te++)
	{
		for (unsigned short in=0; in<colunas; in++)
		{
			if ( (MaxTrain(in) - MinTrain(in))  ==0 )
			{
				EntranceNormalization(te,in)=0;
			}
			else
			{
				EntranceNormalization(te,in) = (2*notNormalized(te,in) - MaxTrain(in) - MinTrain(in)) / (MaxTrain(in) - MinTrain(in));
			}
		}
	}
	return EntranceNormalization;
}

//=================================
mat Normalization::EntranceNormalizationZScore (
	const unsigned short &linhas, 
	const unsigned short &colunas, 
	const mat &notNormalized, 
	const rowvec &MeanTrain, 
	const rowvec &SDevTrain)
{
	mat EntranceNormZScore(linhas,colunas);

	for(unsigned short te=0; te<linhas; te++)
	{
		for (unsigned short in=0; in<colunas; in++)
		{
			EntranceNormZScore(te,in) = ( notNormalized(te,in) - MeanTrain(in) ) / ( SDevTrain(in) );
		}
	}
	return EntranceNormZScore;
}

//=================================
mat Normalization::GlobalNormalizationMinMax (
	const unsigned short &linhas, 
	const unsigned short &colunas, 
	const mat &notNormalized, 
	const rowvec &MaxTrain, 
	const rowvec &MinTrain)
{
	mat GlobalNormalization(linhas,colunas);

	for(unsigned short k=0; k<linhas; k++)
	{
		for (unsigned short j=0; j<colunas; j++)
		{
			if ( (MaxTrain(0) - MinTrain(0))  ==0)
			{
				GlobalNormalization(k,j)=0;
			}
			else
			{
				GlobalNormalization(k,j) = (2.0*notNormalized(k,j) - MaxTrain(0) - MinTrain(0)) / (MaxTrain(0) - MinTrain(0));//if data is yet normalized
			}
		}
	}
	return (GlobalNormalization);
}

//=================================
mat Normalization::GlobalNormalizationZScore (
	const unsigned short &linhas, 
	const unsigned short &colunas, 
	const mat &notNormalized, 
	const rowvec &MeanTrain, 
	const rowvec &SDevTrain)
{
	mat GlobalNormZScore(linhas,colunas);
	for(unsigned short te=0; te<linhas; te++)
	{
		for (unsigned short in=0; in<colunas; in++)
		{
			GlobalNormZScore(te,in) = ( notNormalized(te,in) - MeanTrain(0) ) / ( SDevTrain(0) );
		}
	}
	return (GlobalNormZScore);
}

//=================================
mat  Normalization::Normalization_function (
	const rowvec &P1, 
	const rowvec &P2, 
	const mat &notNormalized, 
	const unsigned short &linhas, 
	const unsigned short &colunas, 
	const NormalizationMethod &NormMethod)
{
	mat NormMatrix (linhas, colunas);
	switch (NormMethod)
	{
		case ENTRANCE_MIN_MAX: 
		{
			//Parameter1 = MaxTrain; Parameter2 = MinTrain;
			NormMatrix = EntranceNormalizationMinMax (linhas, colunas, notNormalized, P1, P2);
			break;
		}
				
		case GLOBAL_MIN_MAX:
		{
			////Parameter1 = MaxTrain; Parameter2 = MinTrain;
			NormMatrix = GlobalNormalizationMinMax (linhas, colunas, notNormalized, P1, P2);
			break;
		}

		case ENTRANCE_ZSCORE:
		{
			//Parameter1 = MeanTrain; Parameter2 = SDevTrain;
			NormMatrix = EntranceNormalizationZScore (linhas, colunas, notNormalized, P1, P2);
			break;
		}

		case GLOBAL_ZSCORE:
		{
			//Parameter1 = MeanTrain; Parameter2 = SDevTrain;
			NormMatrix = GlobalNormalizationZScore (linhas, colunas, notNormalized, P1, P2);
			break;
		}

		case NONE:
		{
			//when the data is already normalized 
			NormMatrix = notNormalized;
			break;
		}
			
		default:
		{
			cout << "ERROR!!! ";
			break;
		}
	}
	return (NormMatrix);
}


//=================================
mat Normalization::ReadAndNorm_function (
	const unsigned short &linhas, 
	const unsigned short &colunas, 
	const string &Filename, 
	const rowvec &MaxTrain, 
	const rowvec &MinTrain, 
	const NormalizationMethod &NormMethod)
{
	mat notNormalized = General_functions::lerFicheiro	(linhas, colunas, Filename);
	mat normalized = Normalization_function (MaxTrain, MinTrain, notNormalized, linhas, colunas, NormMethod);
	return (normalized);
}

//=================================
mat Normalization::GlobalDESNormalizationMinMax (
	const unsigned short &linhas, 
	const unsigned short &colunas, 
	const mat &Normalized, 
	const rowvec &MaxTrain, 
	const rowvec &MinTrain)
{
	mat notNormalized(linhas,colunas);

	for(unsigned short k=0; k<linhas; k++)
	{
		for (unsigned short j=0; j<colunas; j++)
		{
			notNormalized(k,j) = ( (MaxTrain(0) - MinTrain(0)) * Normalized(k,j) + MaxTrain(0) + MinTrain(0) ) / 2.0;
		}
	}
	return (notNormalized);
}


//=================================
mat Normalization::EntranceDESNormalizationMinMax (
	const unsigned short &linhas, 
	const unsigned short &colunas, 
	const mat &Normalized, 
	const rowvec &MaxTrain, 
	const rowvec &MinTrain)
{
	mat notNormalized(linhas,colunas);

	for(unsigned short k=0; k<linhas; k++)
	{
		for (unsigned short j=0; j<colunas; j++)
		{
			notNormalized(k,j) = ( Normalized(k,j) * (MaxTrain(j) - MinTrain(j)) + MaxTrain(j)+MinTrain(j) ) / 2.0;
		}
	}
	
	return (notNormalized);
}

//=================================
mat Normalization::EntranceDESNormalizationZScore (
	const unsigned short &linhas, 
	const unsigned short &colunas, 
	const mat &Normalized, 
	const rowvec &MeanTrain, 
	const rowvec &SDevTrain)
{
	mat notNormalized(linhas,colunas);

	for(unsigned short k=0; k<linhas; k++)
	{
		for (unsigned short j=0; j<colunas; j++)
		{
			notNormalized(k,j) = Normalized(k,j)* SDevTrain(j) + MeanTrain(j);
		}
	}
	return (notNormalized);
}

//=================================
mat Normalization::GlobalDESNormalizationZScore (
	const unsigned short &linhas, 
	const unsigned short &colunas, 
	const mat &Normalized, 
	const rowvec &MeanTrain, 
	const rowvec &SDevTrain)
{
	mat notNormalized(linhas,colunas);

	for(unsigned short k=0; k<linhas; k++)
	{
		for (unsigned short j=0; j<colunas; j++)
		{
			notNormalized(k,j) = Normalized(k,j) * SDevTrain(0)+ MeanTrain(0) ;
		}
	}
	return (notNormalized);
}


//=================================
mat  Normalization::DES_Normalization_function (
	const rowvec &P1, 
	const rowvec &P2, 
	const mat &Normalized, 
	const unsigned short &linhas, 
	const unsigned short &colunas, 
	const NormalizationMethod &NormMethod)
{
	mat Des_NormMatrix (linhas, colunas);
	switch (NormMethod)
	{
		case ENTRANCE_MIN_MAX: 
		{
			//Parameter1 = MaxTrain; Parameter2 = MinTrain;
			Des_NormMatrix = EntranceDESNormalizationMinMax (linhas, colunas, Normalized, P1, P2);
			break;
		}
				
		case GLOBAL_MIN_MAX:
		{
			////Parameter1 = MaxTrain; Parameter2 = MinTrain;
			Des_NormMatrix = GlobalDESNormalizationMinMax (linhas, colunas, Normalized, P1, P2);
			break;
		}

		case ENTRANCE_ZSCORE:
		{
			//Parameter1 = MeanTrain; Parameter2 = SDevTrain;
			Des_NormMatrix = EntranceDESNormalizationZScore (linhas, colunas, Normalized, P1, P2);
			break;
		}

		case GLOBAL_ZSCORE:
		{
			//Parameter1 = MeanTrain; Parameter2 = SDevTrain;
			Des_NormMatrix = GlobalDESNormalizationZScore (linhas, colunas, Normalized, P1, P2);
			break;
		}

		case NONE:
		{
			//when the data is already normalized 
			Des_NormMatrix = Normalized; //ok
			break;
		}
			
		default:
		{
			cout << "ERROR!!! ";
			break;
		}
	}
	return (Des_NormMatrix);
}